@@ -407,8 +407,7 @@ void make_par(EnvI& env, Expression* e) {
             c->id(ASTString(enumName));
           }
         }
-        if (c->id() == env.constants.ids.showDzn ||
-            (c->id() == env.constants.ids.showJSON && typeId > 0 && argtype.bt() == Type::BT_INT)) {
+        if (c->id() == env.constants.ids.showJSON && typeId > 0 && argtype.bt() == Type::BT_INT) {
           c->id(env.constants.ids.show);
         }
       }
@@ -683,10 +682,10 @@ void process_deletions(EnvI& e) {
   }
 
   for (auto& it : e.outputVarOccurrences.itemMap) {
-    std::vector<Item*> keptItems;
+    VarOccurrences::Items keptItems;
     for (auto* iit : it) {
       if (!iit->removed()) {
-        keptItems.push_back(iit);
+        keptItems.insert(iit);
       }
     }
     it = keptItems;
@@ -1514,6 +1513,7 @@ void create_output(EnvI& e, FlatteningOptions::OutputMode outputMode, bool outpu
                   check_output_par_fn(env, crhs);
                 }
                 output_vardecls(env, vdi_copy, rhs);
+                top_down(*this, rhs);
                 vd_followed->e(rhs);
               } else if (reallyFlat == vd_orig ||
                          cannot_use_rhs_for_output(env, vd_followed->e()) ||
