@@ -327,7 +327,11 @@ SolverInstanceBase::Status ChuffedSolverInstance::solve() {
   }
   switch (engine.status) {
     case RESULT::RES_UNK:
-      status = SolverInstanceBase::Status::UNKNOWN;
+      if (engine.solutions > 0) {
+        status = SolverInstanceBase::Status::SAT;
+      } else {
+        status = SolverInstanceBase::Status::UNKNOWN;
+      }
       break;
     case RESULT::RES_SAT:
       status = SolverInstanceBase::Status::SAT;
@@ -400,8 +404,10 @@ void ChuffedSolverInstance::printStatistics() {
   ss.add("randomSeed", so.rnd_seed);
 }
 
+#define CHUFFED_VERSION_TO_STRING(x) #x
+
 std::string ChuffedSolverFactory::getVersion(SolverInstanceBase::Options*) {
-  return CHUFFED_VERSION;
+  return CHUFFED_VERSION_TO_STRING(CHUFFED_VERSION);
 }
 
 SolverInstanceBase::Options* ChuffedSolverFactory::createOptions() { return new ChuffedOptions; }
